% This Matlab program generates graph of reduced-form pi matrix for coint example
% when both b1 and b2 are unknown
%
% Specify true parameter values
truea = [1;-1];
trueb = [0;.10];
trueomega = eye(2);
truechol = inv(chol(trueomega));
%
% Specify control parameters
nitz = 10000;     % nitz is the number of simulations
a = zeros(2,nitz);   % column itz of a contains values of a vector for simulation # itz
b = zeros(2,nitz);   % column itz of b contains values of b vector for simulation # itz
pi11 = zeros(1,nitz);  % row 1 col 1 element of reduced-rank pi
pi12 = zeros(1,nitz);
pi21 = zeros(1,nitz);
pi22 = zeros(1,nitz);
vcount = zeros(1,nitz);

% following lines make sure same random numbers are generated each time
% program is run
dseed = 69323;
randn('state',dseed);

% ====================================================================================
% main control loop begins here
itz = 1;
while itz <= nitz;
%
% Generate data
% y will be a (2 x capt) matrix

capt = 50;
eps = randn(2,capt);
y = zeros(2,1);
ylag = y;
it = 1;
while it <= capt;
    ylag = ylag + trueb*(truea'*ylag) + eps(:,it);
    y = [y ylag];
it = it+1;
end;
dely = y(:,2:capt+1) - y(:,1:capt);  % This is the change in y 
ylag = y(:,1:capt);

% Calculate zeta0
svv = ylag*ylag'/capt;
suu = dely*dely'/capt;
suv = dely*ylag'/capt;
d = inv(svv)*suv'*inv(suu)*suv;
[vectors, values] = eig(d);
if values(1,1) < values(2,2);
    a1 = vectors(:,2);
else;
    a1 = vectors(:,1);
end;
a1 = a1 / sqrt(a1'*svv*a1);
zeta0 = suv*a1*a1';

pi11(1,itz) = zeta0(1,1);
pi12(1,itz) = zeta0(1,2);
pi21(1,itz) = zeta0(2,1);
pi22(1,itz) = zeta0(2,2);
itz = itz+1;
end;
% main control loop ends here
% =========================================================================

% summarize results
disp('pi matrix mean, standard deviations');
mean(pi11(1,:)')
std(pi11(1,:)')
mean(pi12(1,:)')
std(pi12(1,:)')
mean(pi21(1,:)')
std(pi21(1,:)')
mean(pi22(1,:)')
std(pi22(1,:)')


subplot('Position',[0.2 0.78 0.6 0.15]);
[h f xx] = plotdens(pi11(1,:)');
plot(xx,f)
axis([-2,2,0,6])
title('Sampling density of \pi11')

subplot('Position',[0.2 0.52 0.6 0.15]);
[h f xx] = plotdens(pi12(1,:)');
plot(xx,f)
axis([-2,2,0,6])
title('Sampling density of \pi12')

subplot('Position',[0.2 0.29 0.6 0.15]);
[h f xx] = plotdens(pi21(1,:)');
plot(xx,f)
axis([-2,2,0,6])
title('Sampling density of \pi21')

subplot('Position',[0.2 0.05 0.6 0.15]);
[h f xx] = plotdens(pi22(1,:)');
plot(xx,f)
axis([-2,2,0,6])
title('Sampling density of \pi22')

mu11 = mean(pi11');
sig11 = sqrt(mean((pi11.^2)')-mu11^2);
'pi11 mean and std dev'
mu11
sig11

mu12 = mean(pi12');
sig12 = sqrt(mean((pi12.^2)')-mu12^2);
'pi12 mean and std dev'
mu12
sig12

mu21 = mean(pi21');
sig21 = sqrt(mean((pi21.^2)')-mu21^2);
'pi21 mean and std dev'
mu21
sig21

mu22 = mean(pi22');
sig22 = sqrt(mean((pi22.^2)')-mu22^2);
'pi22 mean and std dev'
mu22
sig22
