% This Matlab program controls maximum likelihood estimation for simplest cointegrated case
%
% Specify true parameter values
truea = [1;-1];
trueb = [0;.10];
trueomega = eye(2);
truechol = inv(chol(trueomega));
trueomega
truechol'*trueomega*truechol
%
% Specify control parameters
bcont = 3;    % bcont = 3 means b1 = 0, b2 = trueb2, a1 and a2 unrestricted 
		  % bcont = 1 means b1 = 0, a1 = 1, b2 and a2 unrestricted
nitz = 10000;     % nitz is the number of simulations
a = zeros(2,nitz);   % column itz of a contains values of a vector for simulation # itz
b = zeros(2,nitz);   % column itz of b contains values of b vector for simulation # itz
% following lines make sure same random numbers are generated each time
% program is run
dseed = 69323;
randn('state',dseed);

% ====================================================================================
% main control loop begins here
itz = 1;
while itz <= nitz;
%
% Generate data
% y will be a (2 x capt) matrix

capt = 50;
eps = randn(2,capt);
y = zeros(2,1);
ylag = y;
it = 1;
while it <= capt;
    ylag = ylag + trueb*(truea'*ylag) + eps(:,it);
    y = [y ylag];
it = it+1;
end;
dely = y(:,2:capt+1) - y(:,1:capt);  % This is the change in y 
ylag = y(:,1:capt);

%disp('OLS coefficient estimates and standard errors are as follows');
y1 = dely(2,:)';
x1 = ylag';
pi1 = x1 \ y1;
%disp(pi1);
s1 = x1'*x1;
s1 = inv(s1);
s1 = sqrt(diag(s1));
%disp(s1);


if bcont == 1;
	a(1,itz) = 1;
	a(2,itz) = pi1(2)/pi1(1);
	b(1,itz) = 0;
	b(2,itz) = pi1(1);
elseif bcont == 3;
	a(1,itz) = pi1(1)/trueb(2);
	a(2,itz) = pi1(2)/trueb(2);
	b(1,itz) = 0;
	b(2,itz) = trueb(2);
end;

itz = itz+1;
end;
% main control loop ends here
% =======================================================================


% summarize results
disp('a vector mean, standard deviations, mse, and mae');
mean(a')
std(a')
q = a - truea*ones(1,nitz);
mean(q'.^2)
mean(abs(q'))
disp('b vector mean, standard deviations, mse, and mae');
mean(b')
std(b')
q = b - trueb*ones(1,nitz);
mean(q'.^2)
mean(abs(q'))

if bcont == 3;
subplot('Position',[0.2 0.70 0.6 0.2]);
[h f xx] = plotdens(a(1,:)');
plot(xx,f)
axis([-3,3,0,1])
title('Sampling density of a1')

subplot('Position',[0.2 0.40 0.6 0.2]);
[h f xx] = plotdens(a(2,:)');
plot(xx,f)
axis([-3,3,0,1])
title('Sampling density of a2')

elseif bcont == 1;

subplot('Position',[0.2 0.70 0.6 0.2]);
[h f xx] = plotdens(a(2,:)');
%[f xx] = ksdensity(a(2,:)','npoints',10000);
plot(xx,f)
axis([-5,5,0,1])
title('Sampling density of a2')

subplot('Position',[0.2 0.40 0.6 0.2]);
[h f xx] = plotdens(b(2,:)');
plot(xx,f)
axis([-2,2,0,6])
title('Sampling density of b2')

end;

