function z = ftd_mix2a_post(gtheta,T,yser,gmubar,gmuSigma,galpha,gsigma);
%  Negative logPosterior function with variances fixed -- a 2-state version.
%
% gtheta: 3-by-1.  [mu1, mu2, log((1-p1)/p1)].  Here nStates (number of states) = 2.
% T:  Sample size.
% yser:  T-by-1 observations.
% gmubar: nStates-by-1 prior mean of [mu1,mu2]'.
% gmuSigma: nStates-by-nStates prior covariance of [mu1,mu2]'.
% galpha: nStates-by-1 prior parameters for a Dirichlet prior of p and 1-p.
% gsigma: nstates-by-1 fixed standard devaitions for mu.
%--------
% z:  Negative log posterior.

nStates=length(gmubar);
p = exp(gtheta(3))/(1+exp(gtheta(3)));
z = 0;   % Cummulative for the negative log posterior.
for drawi=1:T
   w1 = p/(sqrt(2*pi)*gsigma(1))*exp( - 0.5*( (yser(drawi)-gtheta(1))^2/gsigma(1)^2 ) );
   w2 = (1-p)/(sqrt(2*pi)*gsigma(2))*exp( - 0.5*( (yser(drawi)-gtheta(2))^2/gsigma(2)^2 ) );

   %**** With a prior on p.
   if ((w1+w2)<1e-320) | (p<1e-320) | ((1-p)<1e-320)
      z = 1e+20;
   else
      z = z - log( w1 + w2 ) ...
            + ( (gtheta(1:2)-gmubar)'/gmuSigma ) * (gtheta(1:2)-gmubar) ...
            - (galpha(1)-1)*log(p) - (galpha(2)-1)*log(1-p);
   end
end


