
#ifndef __STRING_ARRAY__
#define __STRING_ARRAY__

#include <stdio.h>

#define ALL_STRING_ERR  0x0000000F
#define MEM_STRING_ERR  0x00000001
#define SIZE_STRING_ERR 0x00000002
#define NULL_STRING_ERR 0x00000004
#define EOF_STRING_ERR  0x00000008

#define REMOVE_EMPTY_FIELDS 0x00000001
#define STRIP_WHITESPACE    0x00000002

typedef char **TStringVector;
typedef char ***TStringMatrix;

#define V_DIM(x) (((int*)(x))[-1])
#define M_ROW(x) (((int*)(x))[-2])
#define M_COL(x) (((int*)(x))[-1])

int GetStringError(void);
void ClearStringError(void);
int SetStringErrorTerminate(int err);
int SetStringErrorVerbose(int err);

TStringVector CreateStringVector(int m);
TStringMatrix CreateStringMatrix(int m, int n);
void FreeStringVector(TStringVector x);
void FreeStringMatrix(TStringMatrix x);

TStringVector EquateStringVector(TStringVector x, TStringVector y);
TStringMatrix EquateStringMatrix(TStringMatrix x, TStringMatrix y);

/* Reading and writing delimited files */
TStringVector CreateStringVectorFromDelimited(char *buffer, char delimiter, int flag);
TStringVector ReadDelimitedStringVector(FILE *f, char delimiter, int flag);
TStringMatrix ReadDelimitedStringMatrix(FILE *f, char delimiter, int flag);
void PrintDelimitedStringVector(FILE *f, TStringVector x, char delimiter);
void PrintDelimitedStringMatrix(FILE *f, TStringMatrix x, char delimiter);

/* String Manipulation Routines */
char* DuplicateString(char *buffer);
void StripLeadingSpaces(char *buffer);
void StripTrailingSpaces(char *buffer);
void StripSpaces(char *buffer);
int  IsFloat(char *buffer);
int  IsInteger(char *buffer);

#endif
