
/*******************************************************************************
 A matrix or vector array is a array of matrices or vectors with an extra
 sizeof(int) bytes allocated to contain the length of the array.  The length
 of the array can obtained with the MA_LEN or VA_LEN macros.  Matrix and vector
 arrays must be created with calls to CreateMatrixArray or CreateVectorArray
 and freed with calls to FreeMatrixArray or FreeVectorArray.

 The binary layout of a matrix or vector array
      ___ ______ ______       ________
     |   |      |      |     |        |
     | d | p[0] | p[1] | ... | p[d-1] |
     |___|______|______|     |________|

    d    : sizeof(int) bytes containing the length of the array
    p[i] : pointer to the ith matrix or vector in the array
*******************************************************************************/

#ifndef __MATRIX_ARRAY__
#define __MATRIX_ARRAY__

#include "matrix.h"

/* Matrix Arrays */
#define MA_LEN(A) (*(((int *)(A))-1))
TMatrix* CreateMatrixArray(int length);
void FreeMatrixArray(TMatrix *A);

/* Vector Arrays */
#define VA_LEN(A) (*(((int *)(A))-1))
TVector* CreateVectorArray(int length);
void FreeVectorArray(TVector *A);

#endif
