
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include "rand.h"

static long idum=-1;

long initialize_generator(long init)
{
 if (init > 0)
   idum=-init;
  else
   if (init < 0)
     idum=init;
    else
     idum=-time((time_t *)NULL);
 return idum;
}

/*
   The following code is adapted from Numerical Recipes in C
*/
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)
float uniform(void)
{
 long j;
 long k;
 static long iy=0;
 static long iv[NTAB];
 float temp;

 if (idum <= 0 || !iy)
  {
   if (-idum < 1)
     idum=1;
    else
     idum=-idum;
   for (j=NTAB+7; j >= 0; j--)
    {
     k=idum/IQ;
     idum=IA*(idum-k*IQ)-IR*k;
     if (idum < 0) idum+=IM;
     if (j < NTAB) iv[j]=idum;
    }
   iy=iv[0];
  }
 k=idum/IQ;
 idum=IA*(idum-k*IQ)-IR*k;
 if (idum < 0) idum+=IM;
 j=iy/NDIV;
 iy=iv[j];
 iv[j]=idum;
 return ((temp=(float)(AM*iy)) > RNMX) ? (float)RNMX : temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

/*
   Returns a standard gaussian deviate.  The density function for the
   standard gaussian is

                          1
                     ----------- exp(-0.5*x^2)
                      sqrt(2*Pi)

*/
float gaussian(void)
{
 static long iset=0;
 static float gset;
 float fac,r,v1,v2;

 if  (iset == 0)
   {
    do
     {
      v1=(float)(2.0*uniform()-1.0);
      v2=(float)(2.0*uniform()-1.0);
      r=v1*v1+v2*v2;
     }
    while (r >= 1.0);
    fac=(float)sqrt(-2.0*log(r)/r);
    gset=v1*fac;
    iset=1;
    return v2*fac;
	}
  else
   {
    iset=0;
    return gset;
	}
}

/*
   Returns a gamma deviate.  The density function for a gamma distribution is

                              1
                          --------- x^(a-1) exp(-x)
                           gamma(a)

   where here gamma denotes the gamma function, which is defined as the
   integral with respect to t from 0 to infinity of exp(-t)*t^(z-1)

   Uses algorithm from Numerical Recipes in C, which is due to Ahrens and
   described in Knuth.
*
#define PI 3.1415926535897932
/*float gamma(int a)
{
 int i;
 float am, e, s, x, y;

 if (a <= 0) return 0;
 if (a < 6)
  {
   x=1.0;
   for (i=a-1; i >= 0; i--) x*=uniform();
   return (float)(-log(x));
  }

 do
  {
   do
    {
     y=(float)tan(PI*uniform());
     am=(float)(a-1);
     s=(float)sqrt(2.0*am+1.0);
     x=s*y+am;
    }
   while (x <= 0.0);
   e=(float)((1.0+y*y)*exp(am*log(x/am)-s*y));
  }
 while (uniform() > e);
 return x;
}
*/
#undef PI


/*
   Returns the integral from -infinity to x of 1/sqrt(2*PI)*exp(-y^2/2).
   Routine adapted from Numerical Recipes in C.
*/
double cumulative_normal(double x)
{
 double z=fabs(0.7071067811865*x), t=2.0/(2.0+z);

 return (x > 0) ?
           1.0-0.5*t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+
             t*(0.09678418+t*(-0.18628806+t*(0.27886807+t*(-1.13520398+
               t*(1.48851587+t*(-0.82215223+t*0.17087277)))))))))
                :
           0.5*t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+
             t*(0.09678418+t*(-0.18628806+t*(0.27886807+t*(-1.13520398+
               t*(1.48851587+t*(-0.82215223+t*0.17087277)))))))));

}
